@@ -32,38 +32,6 @@ class Missing(object):
 missing = Missing()
 del Missing
 
-class cached_property(object):
-    """
-    A cached property object caches the value returned by the ``getter``.
-
-    :param name:
-        The name which should be used for the property, if you don't pass one
-        the name of the ``getter`` is used.
-
-    :param doc:
-        The docstring which should be used for the property, if no one is
-        passed the ``getter``'s docstring is used.
-    """
-    def __init__(self, getter, name=None, doc=None):
-        self.getter = getter
-        self.__name__ = name or getter.__name__
-        self.__module__ = getter.__module__
-        self.__doc__ = doc or getter.__doc__
-
-    def __get__(self, obj, type=None):
-        missing = object()
-        if obj is None:
-            return self
-        value = obj.__dict__.get(self.__name__, missing)
-        if value is missing:
-            value = self.getter(obj)
-            obj.__dict__[self.__name__] = value
-        return value
-
-    def __repr__(self):
-        return "{0}({1!r}, name={2!r}, doc={3!r})".format(self.getter,
-                                                          self.name, self.doc)
-
 def decode_arguments(arguments,
                      encoding=sys.stdin.encoding or sys.getdefaultencoding()):
     """
@@ -110,15 +78,6 @@ def matches(beginning, strings):
         if string.startswith(beginning):
             yield string
 
-def get_usage(command, callpath):
-    result = [u"usage: {0}".format(u" ".join(map(itemgetter(0), callpath)))]
-    if command.options:
-        result.append(u"[options]")
-    if len(command.commands) > 1 or \
-            command.commands and u"help" not in command.commands:
-        result.append(u"[commands]")
-    return u" ".join(result)
-
 class Node(object):
     """
     Represents an argument passed to your script.
@@ -194,8 +153,10 @@ class Option(Node):
                  short_description=None, long_description=None):
         Node.__init__(self, short_description=short_description,
                       long_description=long_description)
-        self.short = unicode(short)
-        self.long = unicode(long)
+        if short is long is None:
+            raise ValueError("you need to specify a short and/or long")
+        self.short = short if short is None else unicode(short)
+        self.long = long if long is None else unicode(long)
         self.default = default
 
     def evaluate(self, callpath, argument=missing):
@@ -263,10 +224,12 @@ class MultipleOptions(Option):
         'foo,"bar,baz"' -> ["foo", "bar,baz"]
     """
     def __init__(self, sub_option=Option, short=None, long=None,
-                 default=missing, description=None):
+                 default=missing, short_description=None,
+                 long_description=None):
         Option.__init__(self, short=short, long=long, default=default,
-                        description=description)
-        self.sub_option = sub_option()
+                        short_description=short_description,
+                        long_description=long_description)
+        self.sub_option = sub_option(long=u'sub-option')
 
     def evaluate(self, callpath, argument):
         result = []
@@ -279,13 +242,19 @@ class MultipleOptions(Option):
                     buffer = []
                 open_quote = not open_quote
             elif char == u"," and not open_quote:
+                if not buffer:
+                    continue
                 result.append(u"".join(buffer))
                 buffer = []
             else:
                 buffer.append(char)
         if buffer:
             result.append(u"".join(buffer))
-        return map(self.sub_option.evaluate, result)
+        sub_option_cp = callpath + [(u'--sub-option', self.sub_option)]
+        return [
+            self.sub_option.evaluate(sub_option_cp, arg)
+            for arg in result
+        ]
 
 def get_option_attributes(obj):
     return getmembers(obj, lambda x: isinstance(x, Option))
@@ -340,7 +309,7 @@ class Command(Node):
         if takes_arguments is not None:
             self.takes_arguments = takes_arguments
 
-    @cached_property
+    @property
     def short_options(self):
         """
         A dictionary mapping the short variants of the options to a tuple of
@@ -351,7 +320,7 @@ class Command(Node):
             result[option.short] = (name, option)
         return result
 
-    @cached_property
+    @property
     def long_options(self):
         """
         A dictionary mapping the long variants of the options to a tuple of
@@ -368,8 +337,12 @@ class Command(Node):
             result[abbr] = long_options[long_option]
         return result
 
-    @cached_property
+    @property
     def all_commands(self):
+        """
+        A dictionary mapping the command names (including abbreviations) to a
+        tuple of the complete command name and the command itself.
+        """
         commands = dict((k, (k, v)) for k, v in self.commands.iteritems())
         if not self.allow_abbreviated_commands:
             return commands.copy()
@@ -377,30 +350,42 @@ class Command(Node):
             commands[abbr] = commands[command]
         return commands
 
-    def print_missing_node(self, node, callpath, command):
+    def get_usage(self, callpath):
+        result = [u'usage: {0}'.format(u' '.join(map(itemgetter(0), callpath)))]
+        if self.options:
+            result.append(u'[options]')
+        if len(self.commands) > 1 or \
+                self.commands and u'help' not in self.commands:
+            result.append(u'[commands]')
+        return u' '.join(result)
+
+    def print_missing_node(self, node, callpath):
         write = lambda x: callpath[0][1].out_file.write(x + u"\n")
-        write(get_usage(callpath[-1][1], callpath))
-        write(u"")
         if node.startswith(u"-"):
+            write(callpath[-2][1].get_usage(callpath[:1]))
             type = u"option"
-            possible_items = [option.long for option in command.options.values()]
+            possible_items = [option.long for option in self.options.values()]
         else:
+            write(callpath[-1][1].get_usage(callpath[:1]))
             type = u"command"
-            possible_items = command.commands.keys()
-        for shorter_version in shorter(node):
+            possible_items = self.commands.keys()
+        write(u'')
+        for shorter_version in shorter(node.strip(u'-')):
             items = list(matches(shorter_version, possible_items))
             if items:
                 break
         else:
-            items = list(matches(node, possible_items))
+            items = list(matches(node.strip(u'-'), possible_items))
         if not items:
             write(u"The given {0} \"{1}\" does not exist.".format(type, node))
-            return
+            sys.exit(1)
         write(u"The given {0} \"{1}\" does not exist, did you mean?" \
                 .format(type, node))
         for item in items:
+            item = u'-' * node[:2].count(u'-') + item
             write(u" - {0}".format(item))
         write(u"")
+        sys.exit(1)
 
     def evaluate(self, callpath, arguments):
         """
@@ -445,8 +430,7 @@ class Command(Node):
             try:
                 name, option = self.short_options[short]
             except KeyError:
-                self.print_missing_node(u"-" + short, callpath, self)
-                return
+                self.print_missing_node(u"-" + short, callpath)
             callpath[-1] = (callpath[-1][0], option)
             if option.requires_argument:
                 result[name] = option.evaluate(callpath, arguments.next()[1])
@@ -465,8 +449,7 @@ class Command(Node):
         try:
             name, option = self.long_options[long]
         except KeyError:
-            self.print_missing_node(callpath[-1][0], callpath, self)
-            return
+            self.print_missing_node(callpath[-1][0], callpath)
         callpath[-1] = (callpath[-1][0], option)
         used_arguments = []
         if option.requires_argument:
@@ -505,6 +488,13 @@ class HelpCommand(Command):
                          allow_abbreviated_options=allow_abbreviated_options)
 
     def evaluate(self, callpath, arguments):
+        def get_node(node, argument, d):
+            try:
+                node = d[node][1]
+            except KeyError:
+                command.print_missing_node(argument, callpath)
+            return node
+
         command = callpath[-2][1]
         try:
             argument = arguments[0]
@@ -513,36 +503,34 @@ class HelpCommand(Command):
             callpath = callpath[:-1]
         else:
             if argument.startswith(u'--'):
-                try:
-                    node = command.long_options[argument[2:]][1]
-                except KeyError:
-                    self.print_missing_node(argument, callpath, command)
-                    return
-                else:
-                    argument = argument[2:]
+                node = get_node(
+                    argument[2:],
+                    argument,
+                    command.long_options,
+                )
+                argument = argument[2:]
             if argument.startswith(u'-'):
-                try:
-                    node = command.short_options[argument[1:]][1]
-                except KeyError:
-                    self.print_missing_node(argument, callpath, command)
-                    return
-                else:
-                    argument = argument[1:]
+                node = get_node(
+                    argument[1:],
+                    argument,
+                    command.short_options,
+                )
+                argument = argument[1:]
             else:
-                try:
-                    node = command.all_commands[argument][1]
-                except KeyError:
-                    self.print_missing_node(argument, callpath, command)
-                    return
+                node = get_node(
+                    argument,
+                    argument,
+                    command.all_commands,
+                )
             callpath = callpath[:-1] + [(argument, None)]
 
         write = lambda x: callpath[0][1].out_file.write(x + u"\n")
-        write(get_usage(node, callpath))
+        write(node.get_usage(callpath))
         write(u"")
         write(node.long_description)
         write(u"")
         if not isinstance(node, Command):
-            return
+            sys.exit(1)
         commands = sorted(node.commands.items(),
                           key=lambda x: x[1]._position_hint)
         options = sorted(node.options.values(),
@@ -552,23 +540,39 @@ class HelpCommand(Command):
         if commands:
             nodes.append((u"Commands:", get_length(commands), commands))
         if options:
-            commands = [
-                (u"-{0} --{1}".format(o.short, o.long), o) for o in options]
-            nodes.append((u"Commands:", get_length(commands), commands))
+            option_nodes = []
+            for option in options:
+                if option.short is None:
+                    option_nodes.append(
+                        (u'--{0}'.format(option.long), option)
+                    )
+                elif option.long is None:
+                    option_nodes.append(
+                        (u'-{0}'.format(option.short), option)
+                    )
+                else:
+                    option_nodes.append((
+                        u'-{0} --{1}'.format(
+                            option.short, option.long
+                        ),
+                        option
+                    ))
+            nodes.append((u"Options:", get_length(option_nodes), option_nodes))
         for label, max_node_length, nodes in nodes:
             write(label)
             for node_name, node in nodes:
                 write(u" {0} {1}".format(node_name.ljust(max_node_length),
                                          node.short_description))
             write("")
+        sys.exit(1)
 
 class Parser(Command):
-    def __init__(self, options=None, commands=None, script_name=sys.argv[0],
+    def __init__(self, options=None, commands=None, script_name=None,
                  description=None, out_file=sys.stdout, takes_arguments=None):
         Command.__init__(self, options=options, commands=commands,
                          long_description=description,
                          takes_arguments=takes_arguments)
-        self.script_name = script_name
+        self.script_name = sys.argv[0] if script_name is None else script_name
         self.out_file = out_file
 
     @property
